/****************************************************************************
 *
 * Copy right:   Qx.CJ
 * File name:    MainTask.c
 * Description:  主要负责：1.工作模式转换2.定时器定时3.定时保存运行数据4.异常状态重启
 * History:      2021-07-05
 * Version:      V3.0
 ****************************************************************************/
#include "AppTaskMain.h"

static StaticTask_t           gProcess_Main_Task_t;
static UINT8                  gProcess_Main_TaskStack[PROC_MAIN_TASK_STACK_SIZE];
static osThreadId_t           MainTaskId = NULL;
static process_Main 		    gProcess_Main_Task;
#define PROC_MAIN_STATE_SWITCH(a)  (gProcess_Main_Task = a)
#define PROC_APP_STATE_SWITCH(a)  (gProcess_app = a)
static volatile BOOL WorkTimerEnd = FALSE;
static volatile BOOL SleepTimerEnd = FALSE;
void monitor_timer_callback(TimerHandle_t xTimer);
void work_timer_callback(TimerHandle_t xTimer);
void sleep_timer_callback(TimerHandle_t xTimer);

UINT8 MainSlpHandler          = 0xff;//主线程睡眠句柄
static void MainTask(void* arg)
{
    
    TimerHandle_t monitor_timer = NULL;
    TimerHandle_t work_timer = NULL;
    TimerHandle_t sleep_timer = NULL;
    monitor_timer = xTimerCreate("monitor_timer", 100 / portTICK_RATE_MS, pdTRUE, NULL, monitor_timer_callback);
    work_timer    = xTimerCreate("work_timer", 1*60*1000 / portTICK_RATE_MS, pdTRUE, NULL, work_timer_callback);
    sleep_timer   = xTimerCreate("sleep_timer", 2*60*1000 / portTICK_RATE_MS, pdTRUE, NULL, sleep_timer_callback);
    NetSocDisplay(LED_SOC_0,LED_TURN_ON);
	NetSocDisplay(LED_SOC_1,LED_TURN_ON);
	NetSocDisplay(LED_SOC_2,LED_TURN_ON);
	NetSocDisplay(LED_SOC_3,LED_TURN_ON);
	FaultDisplay(LED_TURN_ON);
    PROC_MAIN_STATE_SWITCH(PROCESS_STATE_IDLE);
    slpManSetPmuSleepMode(true,SLP_HIB_STATE,false);
    slpManApplyPlatVoteHandle("MainSlp",&MainSlpHandler);
    slpManPlatVoteDisableSleep(MainSlpHandler, SLP_SLP2_STATE);//禁用SLEEP2休眠
    //配置文件及运行数据加载
    LoadAppConfig();
    LoadAppData();
    //
    xTimerStart(monitor_timer, 0);
    while(true)
    {
        switch(gProcess_Main_Task)
        {
            case PROCESS_STATE_IDLE:
            {
                PROC_APP_STATE_SWITCH(IDLE);
                xTimerStart(work_timer, 0);
                PROC_MAIN_STATE_SWITCH(PROCESS_STATE_WORK);
                break;
            }
            case PROCESS_STATE_WORK:
            {
                osDelay(100);
                PROC_APP_STATE_SWITCH(WORK);
                if(WorkTimerEnd)
                {
                    PROC_MAIN_STATE_SWITCH(PROCESS_STATE_LISTEN);
                    xTimerStop(work_timer,0);
                    xTimerStart(sleep_timer, 0);
                    WorkTimerEnd = FALSE;
                }
                break;
            }
            case PROCESS_STATE_LISTEN:
            {
                osDelay(100);
                PROC_APP_STATE_SWITCH(LISTEN);
                if(AppDataInfo.appDataModify)
                {
                    SaveAppData();
                }
                if(AppNVMData.appDataModify)
                {
                    SaveAppConfig();
                }
                if(SleepTimerEnd)
                {
                    PROC_MAIN_STATE_SWITCH(PROCESS_STATE_IDLE);
                    xTimerStop(sleep_timer,0);
                    SleepTimerEnd = FALSE;
                }
                break;
            }
            default:
            {
                PROC_MAIN_STATE_SWITCH(PROCESS_STATE_IDLE);
                break;
            }
        }
    }
}
void monitor_timer_callback(TimerHandle_t xTimer)
{
    TimeCounter++;
    if(TimeCounter>0xFFFFFFF0)
    {
        TimeCounter=0;
    }
}
void work_timer_callback(TimerHandle_t xTimer)
{
    #ifndef USING_PRINTF
        printf("work_timer_callback\n")
    #endif
    WorkTimerEnd = true;
}
void sleep_timer_callback(TimerHandle_t xTimer)
{
    #ifndef USING_PRINTF
        printf("sleep_timer_callback\n")
    #endif
    SleepTimerEnd = true;
}
void AppTaskMainInit(void *arg)
{
    #ifndef USING_PRINTF
    if(BSP_GetPlatConfigItemValue(PLAT_CONFIG_ITEM_LOG_CONTROL) != 0)
    {
        HAL_UART_RecvFlowControl(false);
    }
    #endif
    osThreadAttr_t task_attr;
    memset(&task_attr,0,sizeof(task_attr));
    memset(gProcess_Main_TaskStack, 0xA5, PROC_MAIN_TASK_STACK_SIZE);
    task_attr.name = "Main_Task";
    task_attr.stack_mem = gProcess_Main_TaskStack;
    task_attr.stack_size = PROC_MAIN_TASK_STACK_SIZE;
    task_attr.priority = osPriorityNormal;
    task_attr.cb_mem = &gProcess_Main_Task_t;
    task_attr.cb_size = sizeof(StaticTask_t);
    MainTaskId = osThreadNew(MainTask, NULL, &task_attr);

}
void AppTaskMainDeInit(void *arg)
{
    osThreadTerminate(MainTaskId);
    MainTaskId = NULL;
}

