/****************************************************************************
 *
 * Copy right:   Qx.
 * File name:    UartTask.c
 * Description:  串口任务
 * History:      2021-03-05
 *
 * 2021-04-11:可以通过Ota线路升级Bms保护板
 ****************************************************************************/

#include "AppTaskUart.h"

static StaticTask_t             gProcess_Uart_Task_t;
static UINT8                  gProcess_Uart_TaskStack[PROC_UART_TASK_STACK_SIZE];
static osThreadId_t           UartTaskId = NULL;
static process_Uart             gProcess_Uart_Task = PROCESS_UART_STATE_IDLE;
#define PROC_UART_STATE_SWITCH(a)  (gProcess_Uart_Task = a)
extern ARM_DRIVER_USART Driver_USART1;
static ARM_DRIVER_USART *USARTdrv = &Driver_USART1;
volatile BOOL isRecvTimeout = false;
volatile BOOL isRecvComplete = false;

void USART_callback(uint32_t event);
static UINT16 crc_chk(UINT8* data, UINT8 length);
UINT8 Uart_Encrypt_Send(void);
static UINT8 Uart_DataRecv_func(UartQueryType Uart_Read_Msg_Fun,UINT8* Uart_Recv_Buffer_Fun);
static void UartTask(void* arg)
{
    USARTdrv->Initialize(USART_callback);
    USARTdrv->PowerControl(ARM_POWER_FULL);
    USARTdrv->Control(ARM_USART_MODE_ASYNCHRONOUS |
                      ARM_USART_DATA_BITS_8 |
                      ARM_USART_PARITY_NONE |
                      ARM_USART_STOP_BITS_1 |
                      ARM_USART_FLOW_CONTROL_NONE, 9600);
    if(UartWriteCmdHandle == NULL)//Uart控制命令传输指针
	{
		UartWriteCmdHandle = osMessageQueueNew(3,sizeof(UartWriteData_S), NULL);
	}
    UINT16  Uart_Uds_LEN;
    UINT16  Reg_Num = 0;
    UINT16  Uart_Recv_LEN;
    UartQueryType Uart_Read_Msg;//发送结构体初始化
    memset(&(Uart_Read_Msg),0x00,sizeof(Uart_Read_Msg));
    UartAnswerData_S UartAnswerData;//应答数据初始化
	memset(&(UartAnswerData),0x00,sizeof(UartAnswerData_S));
    UartWriteData_S UartWriteData; //Uart控制命令初始化
	memset(&(UartWriteData),0x00,sizeof(UartWriteData_S));
    PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_ENCRYPT);
    while (1)
    {
        switch (gProcess_Uart_Task)
        {
            case PROCESS_UART_STATE_ENCRYPT:
            {
                UINT8 EncryptFlag=0x00;
				UINT8 EncryptCount=0;
				while(EncryptFlag!=0x01&&EncryptCount<=3)
				{
					EncryptFlag = Uart_Encrypt_Send();
					EncryptCount++;
				}
                PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_IDLE);
                break;
            }
            case PROCESS_UART_STATE_IDLE:
            {
                osDelay(100);
                if(TimeCounter%10==0 && gProcess_app==WORK)
                {
                    if(osMessageQueueGet(UartWriteCmdHandle,&UartWriteData,0,0)==osOK)
                    {
                        PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_WRITE);
                    }
                    else
                    {
                        PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_READ);
                    }
                }
                else if (gProcess_app==LISTEN)
                {
                    PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_SLEEP);
                    break;
                }
                if(AppDataInfo.BattLock==TRUE && ((UartAnswerData.data[(0x09 + AppNVMData.BattCellCount + AppNVMData.BattTempCount + BMS_OTHER_TEMP)*2+1])>>1)&0x03!=0x00 && TimeCounter%10==0)//try to lock
				{
					UartWriteData.WriteCmd = 0x01;
					UartWriteData.Data[0] = 0x00;
					UartWriteData.Data[1] = 0x00;
					osMessageQueuePut(UartWriteCmdHandle,&UartWriteData,0,1000);
					PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_READ);
				}
				else if (AppDataInfo.BattLock==FALSE && ((UartAnswerData.data[(0x09 + AppNVMData.BattCellCount + AppNVMData.BattTempCount + BMS_OTHER_TEMP)*2+1])>>1)&0x03==0x00 && TimeCounter%10==0 ) // try to unlock
				{
					UartWriteData.WriteCmd = 0x01;
					UartWriteData.Data[0] = 0x00;
					UartWriteData.Data[1] = 0x03;
					osMessageQueuePut(UartWriteCmdHandle,&UartWriteData,0,1000);
					PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_READ);
				}
                if(BMS_Fota_update_flag)
                {
					PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_UPDATE);
                    break;
                }
                break;
            }
            case PROCESS_UART_STATE_READ:
            {
                UINT16 CRC_chk_buffer;
                Reg_Num = 0x21 + AppNVMData.BattCellCount + AppNVMData.BattTempCount + BMS_OTHER_TEMP ;//按照协议里面的0x21+X+N的结束地址
                Uart_Read_Msg.Bms_Address = BMS_ADDRESS_CODE;
                Uart_Read_Msg.Bms_Funcode = UART_READ_CODE;
                Uart_Read_Msg.Reg_Begin_H = 0x00;
                Uart_Read_Msg.Reg_Begin_L = 0x00;
                Uart_Read_Msg.Reg_Num_H   = Reg_Num>>8;
                Uart_Read_Msg.Reg_Num_L   = Reg_Num;
                Uart_Uds_LEN = Reg_Num*2;
                memset(UartAnswerData.Header,0x00,Uart_Uds_LEN);
				CRC_chk_buffer = crc_chk((UINT8 *)&Uart_Read_Msg,6);
				Uart_Read_Msg.CRC_L = CRC_chk_buffer;
				Uart_Read_Msg.CRC_H = CRC_chk_buffer>>8;
				Uart_Recv_LEN = Uart_DataRecv_func(Uart_Read_Msg,(UINT8*)(UartAnswerData.Header));
                #ifdef USING_PRINTF
					printf("[%d]Uart_Recv_buffer-%d: ",__LINE__,Uart_Recv_LEN);
                #endif
                if(Uart_Recv_LEN>0)
                {
                    UartRecvFlag = 1;
                    UartAnswerData.len = Uart_Recv_LEN;
                    uartBattInfoDecode(UartAnswerData.data);
                }
                else
                {
                    UartRecvFlag = 0;
                }

                PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_IDLE);
                #ifdef USING_PRINTF
					printf("[%d]Uart_Recv_buffer-%d: ",__LINE__,Uart_Recv_LEN);
					for(int i=0;i<Uart_Recv_LEN;i++)
					{
						printf("%x ",*((UINT8 *)&UartAnswerData.Header+i));
					}
					printf("\n");
				#endif
                break;
            }
            case PROCESS_UART_STATE_WRITE:
            {

                PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_IDLE);
                break;
            }
            case PROCESS_UART_STATE_UPDATE:
            {

                PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_IDLE);
                break;
            }
            case PROCESS_UART_STATE_SLEEP:
            {
                USARTdrv->PowerControl(ARM_POWER_OFF);
                #ifdef USING_PRINTF
                    printf("Uart silence begin\n");
                #endif
                while (gProcess_app==LISTEN)
                {
                    osDelay(1000);
                }
                PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_ENCRYPT);
                USARTdrv->Initialize(USART_callback);
                USARTdrv->PowerControl(ARM_POWER_FULL);
                USARTdrv->Control(ARM_USART_MODE_ASYNCHRONOUS |
                                ARM_USART_DATA_BITS_8 |
                                ARM_USART_PARITY_NONE |
                                ARM_USART_STOP_BITS_1 |
                                ARM_USART_FLOW_CONTROL_NONE, 9600);
                break;
            }
            default:
            {
                PROC_UART_STATE_SWITCH(PROCESS_UART_STATE_IDLE);
                break;
            }
        }
    }
}
//Uart发送接收函数
static UINT8 Uart_DataRecv_func(UartQueryType Uart_Read_Msg_Fun,UINT8* Uart_Recv_Buffer_Fun)
{
    UINT16 CRC_Rece_buffer = 0x00;
    UINT16 CRC_chk_buffer = 0xff;
    UINT16 Data_Len = 0 ;
    UINT8 timeout = 0x00;
	UINT8 pSendCmd[8];
	memcpy(pSendCmd,(UINT8*)(&Uart_Read_Msg_Fun),8);
	Data_Len  = ((Uart_Read_Msg_Fun.Reg_Num_H<<8)|(Uart_Read_Msg_Fun.Reg_Num_L))*2+5;
	USARTdrv->Send(pSendCmd,8);
    USARTdrv->Receive(Uart_Recv_Buffer_Fun,Data_Len);
	while(true)
    {
        timeout++;
        if((isRecvTimeout == true) || (isRecvComplete == true))
        {
            break;
        }
        else
        {
            osDelay(100);
            if (timeout>=10)
            {
                timeout =0;
                isRecvTimeout = true;
            }
        } 
    }
    if (isRecvComplete == true)
    {
        isRecvComplete = false;
        CRC_Rece_buffer =*(Uart_Recv_Buffer_Fun+Data_Len-1)<<8|*(Uart_Recv_Buffer_Fun+Data_Len-2);
        CRC_chk_buffer = crc_chk(Uart_Recv_Buffer_Fun,Data_Len-2);
	    #ifdef USING_PRINTF1
	         printf("Uart_Rece_buffer after Crc: ");
	         for(int i=0;i<Data_Len;i++)
	         {
	         	printf("%x ",*(Uart_Recv_Buffer_Fun+i));
	   	     }
	         printf("\tcrcchk:%x,%x\n ",CRC_chk_buffer,CRC_Rece_buffer);
	    #endif
        if (CRC_Rece_buffer == CRC_chk_buffer)//满足校验
        {
            return Data_Len;
        }
        else //接收数据的校验不过
        {
            USARTdrv->Uninitialize();
            osDelay(1000);
            USARTdrv->Initialize(USART_callback);
            USARTdrv->PowerControl(ARM_POWER_FULL);
            USARTdrv->Control(ARM_USART_MODE_ASYNCHRONOUS |
                      ARM_USART_DATA_BITS_8 |
                      ARM_USART_PARITY_NONE |
                      ARM_USART_STOP_BITS_1 |
                      ARM_USART_FLOW_CONTROL_NONE, 9600);
            memset(Uart_Recv_Buffer_Fun,0xff,Data_Len);
            return 0;
        }
    }
    else
    {
        memset(Uart_Recv_Buffer_Fun,0x00,Data_Len);
        isRecvTimeout = false;
        return 0;
    }
	return 0;
}
UINT8 Uart_Encrypt_Send()
{
	UINT8 SeedNumberArrray[4]={0x38,0x56,0xfe,0xac};
	UINT16 EncodeNumberArray[4];
	UINT8 UartEncryptBuffer[17];
	UINT8 UartDecryptBuffer[5];
	UINT16 CRC_chk_buffer;
	UINT8 timeCount = 0;
	UartEncryptBuffer[0] = BMS_ADDRESS_CODE;
	UartEncryptBuffer[1] = UART_ENCRYPT_CODE;
	UartEncryptBuffer[2] = 0x0c;
	for(int i=0;i<4;i++)
	{
		SeedNumberArrray[i]=rand();
		EncodeNumberArray[i] = encryptionAlgorithm(SeedNumberArrray[i]);
		UartEncryptBuffer[i+3] = SeedNumberArrray[i];
		UartEncryptBuffer[i*2+7] = EncodeNumberArray[i]>>8;
		UartEncryptBuffer[i*2+8] = EncodeNumberArray[i];
	}
	CRC_chk_buffer = crc_chk(UartEncryptBuffer,17-2);
	UartEncryptBuffer[15] = CRC_chk_buffer;
	UartEncryptBuffer[16] = CRC_chk_buffer>>8;
	USARTdrv->Send(UartEncryptBuffer,17);
	USARTdrv->Receive(UartDecryptBuffer,5); 
	while((isRecvTimeout == false) && (isRecvComplete == false))
	{
		timeCount++;
		osDelay(100);
		if (timeCount>=10)
		{
			timeCount =0;
			isRecvTimeout = true;
			break;
		}
	}
	#ifdef USING_PRINTF
		printf("[%d]Uart_Encrypt_Recv: ",__LINE__);
		for(int i=0;i<5;i++)
		{
		printf("%x ",UartDecryptBuffer[i]);
		}
    #endif	
	if (isRecvComplete == true)
	{
		isRecvComplete = false;
		return UartDecryptBuffer[2];
	}
	else
	{
		isRecvTimeout = false;
		return 0x03;
	}
}
void USART_callback(uint32_t event)
{
    if(event & ARM_USART_EVENT_RX_TIMEOUT)
    {
        isRecvTimeout = true;
    }
    if(event & ARM_USART_EVENT_RECEIVE_COMPLETE)
    {
        isRecvComplete = true;
    }
}
static UINT16 crc_chk(UINT8* data, UINT8 length)
{  
    UINT8 j;
    UINT16 reg_crc=0xFFFF;
    while(length--)
    { 
        reg_crc ^= *data++;
        for(j=0;j<8;j++)
        { 
            if(reg_crc & 0x01)
            {
                reg_crc=(reg_crc>>1) ^ 0xA001;
            }
            else
            {
                reg_crc=reg_crc >>1;
            }
        }
    }
    return reg_crc;
}
void AppTaskUartInit(void *arg)
{
    osThreadAttr_t task_attr;
    memset(&task_attr,0,sizeof(task_attr));
    memset(gProcess_Uart_TaskStack, 0xA5, PROC_UART_TASK_STACK_SIZE);
    task_attr.name = "Uart_Task";
    task_attr.stack_mem = gProcess_Uart_TaskStack;
    task_attr.stack_size = PROC_UART_TASK_STACK_SIZE;
    task_attr.priority = osPriorityBelowNormal7;
    task_attr.cb_mem = &gProcess_Uart_Task_t;
    task_attr.cb_size = sizeof(StaticTask_t);
    UartTaskId = osThreadNew(UartTask, NULL, &task_attr);
}
void AppTaskUartDeInit(void *arg)
{
    osThreadTerminate(UartTaskId);
    UartTaskId = NULL;
}